using System;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Threading;

class Client
{
	const string SERVER_HOST = "127.0.0.1";
	const int SERVER_PORT = 2222;

	static void Main(string[] args)
	{
		TcpClient tcpClient = new TcpClient();
		try
		{
			tcpClient.Connect(SERVER_HOST, SERVER_PORT);
		}
		catch (SocketException)
		{
			Console.WriteLine("Can not connect to the server.");
			Environment.Exit(-1);
		}

		NetworkStream ns = tcpClient.GetStream();
		using (ns)
		{
			StreamReader reader = new StreamReader(ns);
			TextTransmitter serverToConsole = 
				new TextTransmitter(reader, Console.Out);
			Thread serverToConsoleThread = new Thread(
				new ThreadStart(serverToConsole.Transmit));
			serverToConsoleThread.Start();

			StreamWriter writer = new StreamWriter(ns);
			TextTransmitter consoleToServer = 
				new TextTransmitter(Console.In, writer);
			Thread consoleToServerThread = new Thread(
				new ThreadStart(consoleToServer.Transmit));
			consoleToServerThread.Start();

			serverToConsoleThread.Join();
			consoleToServerThread.Join();
		}
	}
}

class TextTransmitter
{
	const int BUFFER_SIZE = 1024;
	
	private TextReader mReader;
	private TextWriter mWriter;

	public TextTransmitter(TextReader aReader, TextWriter aWriter)
	{
		mReader = aReader;
		mWriter = aWriter;
	}

	public void Transmit()
	{
		char[] buf = new char[BUFFER_SIZE];
		while (true)
		{
			int charsRead = mReader.Read(buf, 0, buf.Length);
			if (charsRead == 0)
			{
				// End of the stream reached --> the socket is closed
				Environment.Exit(-1); 
			}
			mWriter.Write(buf, 0, charsRead);
			mWriter.Flush();
		}
	}
}