using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;

class ThreadedTcpServer
{
	const int LISTENING_PORT = 2222;

	public static void Main()
	{
		IPEndPoint serverEndPoint =
			new IPEndPoint(IPAddress.Any, LISTENING_PORT);
		TcpListener server = new TcpListener(serverEndPoint);
		server.Start();
		Console.WriteLine("Server started listening on {0}...",
			server.LocalEndpoint);

		while(true)
		{
			TcpClient client = server.AcceptTcpClient();
			ClientThread clientThread = new ClientThread(client);
			Thread thread = new Thread(
				new ThreadStart(clientThread.ServeClient));
			thread.Start();
		}
	}
}

class ClientThread
{
	const int SOCKET_TIMEOUT = 10 * 1000; // 10 seconds

	private TcpClient mTcpClient;
	private NetworkStream mNetworkStream;
	private StreamReader mReader;
	private StreamWriter mWriter;

	public ClientThread(TcpClient aTcpClient)
	{
		mTcpClient = aTcpClient;
		mTcpClient.ReceiveTimeout = SOCKET_TIMEOUT;
		mNetworkStream = mTcpClient.GetStream();
		mReader = new StreamReader(mNetworkStream);
		mWriter = new StreamWriter(mNetworkStream);
	}

	public void ServeClient()
	{
		Console.WriteLine("Accepted client connection.");
		try
		{
			SendText("Enter your name: ");
			string name = ReceiveLine();
			string response = "Hello, " + name + "\n";
			SendText(response);
		} 
		catch (SendReceiveException sre)
		{
			Console.WriteLine(sre.Message);
		}
		finally
		{
			mNetworkStream.Close();
			mTcpClient.Close();
			Console.WriteLine("Client connection closed.");
		}
	}

	private void SendText(string aText)
	{
		try
		{
			mWriter.Write(aText);
			mWriter.Flush();
			Console.WriteLine("Sent: {0}", aText.Trim('\n'));
		}
		catch (IOException ioex)
		{
			throw new SendReceiveException(
				"Error: Can not send data to the client.", ioex);
		}
	}

	private string ReceiveLine()
	{
		try
		{
			string line = mReader.ReadLine();
			if (line == null)
			{
				throw new SendReceiveException(
					"Error: Connection closed by the client.", null);
			}
			Console.WriteLine("Received: {0}", line);
			return line;
		}
		catch (IOException ioex)
		{
			throw new SendReceiveException(
				"Error: Can not receive data from the client.", ioex);
		}
	}
}

class SendReceiveException : ApplicationException
{
	public SendReceiveException(String aMessage, Exception aCause) : 
		base(aMessage, aCause)
	{
	}
}
